
#include <iostream>
#include <sstream>
#include <fstream>
#include <map>
#include <vector>
#include <iomanip>
using namespace std;

class CDS{
public:
	CDS() {start = stop = phase = complt = 0; strand = '+';}
	int start, stop, phase, complt;
	char strand;
	bool operator <( const CDS& cds )const
	{
		return stop < cds.stop;
	}
};

int main(
		 int argc, char* argv[]
		 )
{
	if(argc != 3)
	{
		cout << "Usage: " << argv[0] << "  mga(in)  metaLoc(out)" << endl;
		return 1;
	}
	string mga_file = argv[1];	// ".\\Release\\WASECA_FRM_SOIL_TRACE_SMPL_20010901.mga";
	std::ifstream fin(mga_file.data());
	if(!fin){cout<<"can't read "<<mga_file<<endl; return 1;}

	string loc_file = argv[2];	//"metaLoc.txt";
	std::ofstream fout;
	fout.open(loc_file.data());
	if(!fout){cout<<"can't write "<<loc_file<<endl; return 1;}

	int shotgun= 0;
	int hasCDS = 0;
	int cds_n = 0;
	int cds90 = 0;
	string line;
	std::map<string, std::vector<CDS> > pid_CDSs;
	getline(fin, line);
	while(!fin.eof()) {
		if(line.find('#') != std::string::npos
		&& line.find("# gc =") == std::string::npos
		&& line.find("# self") == std::string::npos) {
			++ shotgun;
			int lpos = (int)line.find('#')+2;
			//int rpos = (int)line.find(' ', 3);
			string pid = line.substr(lpos);//, rpos-lpos);

			// find the start of annotation of locations
			while(line.find("# self") == std::string::npos && !fin.eof())
				getline(fin, line);
			getline(fin, line);

			bool cds_exist = false;
			while(line.find("gene_") != std::string::npos && !fin.eof()) {
				cds_exist = true;
				++ cds_n;
				CDS cds;
				string w;
				istringstream is(line);
				is >> w >> cds.start >> cds.stop >> cds.strand >> cds.phase >> cds.complt;
				int startCmplt = cds.complt / 10;
				int stopCmplt = cds.complt % 10;
				
				// adapt the location to 3-times length which translates to a amino-acid sequence
				int start = cds.start, stop = cds.stop;
				if(startCmplt == 0 && stopCmplt == 1) {
					if(cds.strand == '+')
						cds.start += cds.phase;
					else
						cds.stop -= cds.phase;
				}
				else if(startCmplt == 1 && stopCmplt == 0) {
					if(cds.strand == '+')
						cds.stop = cds.start + (cds.stop - cds.start+1)/3*3 -1;
					else
						cds.start = cds.stop +1 - (cds.stop - cds.start+1)/3*3;
				}
				else if(startCmplt == 0 && stopCmplt == 0) {
					if(cds.strand == '+') {
						cds.start += cds.phase;
						cds.stop = cds.start + (cds.stop - cds.start+1)/3*3 -1;
					}
					else {
						cds.stop -= cds.phase;
						cds.start = cds.stop +1 - (cds.stop - cds.start+1)/3*3;
					}
				}

				if(cds.stop-cds.start+1>90)	{	// take only CDS longer than 90bps into account as we did
					++cds90;
					std::map<string, vector<CDS> >::iterator iter = pid_CDSs.find(pid);
					if(iter != pid_CDSs.end())
						iter->second.push_back(cds);
					else
						pid_CDSs[ pid ].push_back(cds);
				}

				getline(fin, line);
			}
			if(cds_exist)
				++hasCDS;
		}
	}
	fin.close();
	cout << shotgun << " shotgun sequence reads in which " << hasCDS << " were predicted to be exist CDSs." << endl;

	// output fasta format
	std::map<string, vector<CDS> >::iterator it = pid_CDSs.begin();
	for(; it != pid_CDSs.end(); ++ it) {
		fout << '>' << it->first << endl;
		for(int k = 0; k < (int)it->second.size(); ++k) {
			CDS c = it->second[k];
			fout << std::setw(8) << c.start << std::setw(8) << c.stop << std::setw(4) << c.strand << endl;
				//<< std::setw(4) << c.strand << std::setw(4) << c.phase
				//<< std::setw(3) << c.complt/10 << c.complt%10 << endl;
		}
	}
	cout << "Predict CDS No.\t" << cds_n << endl;
}
